import json
from .condition import Condition
from .documents import Documents

class Request:
    
    def __init__(self):
        self.request = {
            "response" : {
                "documents": {}
            }
        }
        
    def __repr__(self):
        return(json.dumps(self.request))
        
    def raw(self): 
        """Get the request object"""
        return(self.request)
    
    def cache(self, cache = None): 
        """Get and optionally change current setting cache"""
        try:
            currentCache = self.request["cache"]
        except Exception:
            currentCache = None
        if type(cache) is bool:
            self.request["cache"] = cache
        elif not (cache is None):
            raise Exception("cache must be of type bool") 
        return(currentCache)
    
    def condition(self, condition = None):
        """Get and optionally change current condition"""
        try:
            currentCondition = self.request["condition"]
        except Exception:
            currentCondition = None
        if isinstance(condition, Condition):
            self.request["condition"] = condition.raw()
        elif not (condition is None):
            raise Exception("invalid condition") 
        return(currentCondition)
    
    def documents(self, documents = None):
        """Get and optionally change current documents"""
        try:
            currentDocuments = self.request["response"]["documents"]
        except Exception:
            currentDocuments = None
        if isinstance(documents, Documents):
            self.request["response"]["documents"] = documents.raw()
        elif not (documents is None):
            raise Exception("invalid condition") 
        return(currentDocuments)




class Condition:
    
    def __init__(self):
        self.type = None
        self.allowNot = None
        self.allowFacetquery = None
        self.allowExpansion = None
        self.condition = {
        }  
        
    def type(self):
        return(self.type) 
    
    def raw(self): 
        """Get the condition object"""
        if self.type is None:
            raise Exception("no (valid) condition")
        self.condition["type"] = self.type
        return(self.condition)
    
    def param_not(self, value):
        """Negation of the condition"""
        if self.allowNot:
            try:
                currentNot = self.condition["not"]
            except Exception:
                currentNot = None
            if type(value) is bool:
                self.condition["not"] = value
            elif not (value is None):
                raise Exception("value must be a boolean")
            return(currentNot)
        else:
            raise Exception("negation not implemented for this condition")
        
    def param_expansion(self, expansion):
        """Applied expansion for the condition"""
        if self.allowExpansion:
            try:
                currentExpansion = self.condition["expansion"]
            except Exception:
                currentExpansion = None
            if type(expansion) is Expansion:
                self.condition["expansion"] = expansion.raw()
            elif not (expansion is None):
                raise Exception("value must be an expansion")
            return(currentExpansion)
        else:
            raise Exception("expansion not implemented for this condition")    
        
    def param_facetquery(self, value):
        """Create facetquery from condition"""
        if self.allowFacetquery:
            #get current
            try:
                currentFacetquery = self.condition["facetquery"]
            except Exception:
                currentFacetquery = None
            try:
                currentKey = self.condition["key"]
            except Exception:
                currentKey = None   
            #update
            if type(value) is bool:
                self.condition["facetquery"] = value
                #remove previous key    
                try:
                    del self.condition["key"]
                except:
                    pass
            elif type(value) is str:
                self.condition["facetquery"] = True
                self.condition["key"] = value
            elif not(value is None):
                raise Exception("value must be a boolean or string")
            if not (currentKey is None):
                return currentKey
            else:
                return currentFacetquery
        else:
            raise Exception("negation not implemented for this condition")       
        
class ConditionEquals(Condition):
    
    def __init__(self, field, value):
        super().__init__()
        self.type = "equals"
        self.allowNot = True
        self.allowFacetquery = True
        self.allowExpansion = True
        if not(type(field) is str):
            raise Exception("field must be a string")
        self.condition["field"] = field
        if not ((type(value) is str) | (type(value) is bool) | (type(value) is list)):
            raise Exception("value must be a string, boolean or list of strings or booleans")
        self.condition["value"] = value
        
class ConditionPhrase(Condition):
    
    def __init__(self, field, value):
        super().__init__()
        self.type = "phrase"
        self.allowNot = True
        self.allowFacetquery = True
        self.allowExpansion = True
        if not(type(field) is str):
            raise Exception("field must be a string")
        self.condition["field"] = field
        if not ((type(value) is str) | (type(value) is list)):
            raise Exception("value must be a string or list of strings")
        self.condition["value"] = value        
 
class ConditionWildcard(Condition):
    
    def __init__(self, field, value):
        super().__init__()
        self.type = "wildcard"
        self.allowNot = True
        self.allowFacetquery = True
        if not(type(field) is str):
            raise Exception("field must be a string")
        self.condition["field"] = field
        if not ((type(value) is str) | (type(value) is list)):
            raise Exception("value must be a string or list of strings")
        self.condition["value"] = value        

class ConditionRegexp(Condition):
    
    def __init__(self, field, value):
        super().__init__()
        self.type = "regexp"
        self.allowNot = True
        self.allowFacetquery = True
        if not(type(field) is str):
            raise Exception("field must be a string")
        self.condition["field"] = field
        if not ((type(value) is str) | (type(value) is list)):
            raise Exception("value must be a string or list of strings")
        self.condition["value"] = value        

class ConditionRange(Condition):
    
    def __init__(self, field, start, end):
        super().__init__()
        self.type = "range"
        self.allowNot = True
        self.allowFacetquery = True
        if not(type(field) is str):
            raise Exception("field must be a string")
        self.condition["field"] = field        
        if not(start is None):
            if not ((type(start) is str) ):
                raise Exception("start must be a string")
            else:
                self.condition["start"] = start   
        if not(end is None):
            if not ((type(end) is str) ):
                raise Exception("end must be a string")
            else:
                self.condition["end"] = end            


class ConditionCQL(Condition):
    
    def __init__(self, field, value):
        super().__init__()
        self.type = "cql"
        self.allowNot = True
        self.allowFacetquery = True
        if not(type(field) is str):
            raise Exception("field must be a string")
        self.condition["field"] = field
        if not ((type(value) is str) | (type(value) is list)):
            raise Exception("value must be a string or list of strings")
        self.condition["value"] = value      
        
class Expansion:
    
    def __init__(self, value, parameters):
        if not(type(value) is str):
            raise Exception("value must be a string")
        self.expansion = {
            "type" : value
        }  
        if type(parameters) is dict:
            self.expansion["parameters"] = parameters       
        elif not(parameters is None):
            raise Exception("parameters must be a dict")
        
    def raw(self): 
        """Get the expansion object"""
        return(self.expansion)        
            


import requests
import json
from .request.request import Request
from .response.response import Response

class Broker:
    
    def __init__(self, url):
        #define url
        self.url = url
        self.timeout = 180
        
    def do_request(self, request):    
        """Perform a BrokerRequest"""
        if type(request) == Request:
            data = json.dumps(request.raw())  
        elif type(request) == dict:
            data = json.dumps(request)  
        elif type(request) == str:
            data = request
        else:
            raise Exception("unsupported type of request")  
        try: 
            response = requests.post(self.url, data, timeout=self.timeout)
            response.raise_for_status() 
            return(Response(response.text))                        
        except requests.exceptions.HTTPError as httpErr: 
            print ("Http Error:",httpErr) 
        except requests.exceptions.ConnectionError as connErr: 
            print ("Error Connecting:",connErr) 
        except requests.exceptions.Timeout as timeOutErr: 
            print ("Timeout Error:",timeOutErr) 
        except requests.exceptions.RequestException as reqErr: 
            print ("Something Else:",reqErr) 
        raise Exception("no (valid) response from broker")
        
        




class Documents:
    
    def __init__(self, start, rows):
        self.documents = {
        } 
        if type(start)==int:
            self.documents["start"] = start
        elif not(start is None):
            raise Exception("start must be an integer")
        if type(rows)==int:
            self.documents["rows"] = rows
        elif not(rows is None):
            raise Exception("rows must be an integer")   
            
        
    def raw(self): 
        """Get the documents object"""
        return(self.documents)
    
    def param_start(self, value):
        """start of documents"""
        try:
            currentStart = self.documents["start"]
        except Exception:
            currentStart = None
        if type(value) is int:
            self.documents["start"] = value
        elif not (value is None):
            raise Exception("value must be an integer")
        return(currentStart)
    
    def param_rows(self, value):
        """rows of documents"""
        try:
            currentRows = self.documents["rows"]
        except Exception:
            currentRows = None
        if type(value) is int:
            self.documents["rows"] = value
        elif not (value is None):
            raise Exception("value must be an integer")
        return(currentRows)
    
    def param_fields(self, value):
        """fields of documents"""
        try:
            currentFields = self.documents["fields"]
        except Exception:
            currentFields = None
        if type(value) is list:
            self.documents["fields"] = value
        elif not (value is None):
            raise Exception("value must be a list")
        return(currentFields)
    
        



from setuptools import setup, find_packages


setup(
    name="textexploration",
    version="0.2.0",
    author="Matthijs Brouwer",
    packages=find_packages(),
    license="LICENSE.txt",
    long_description=open("README.txt").read(),
    install_requires=[
        "requests >= 2.21.0",
        "pandas",
    ],
)




import json
import pandas as pd

class Response:
    
    def __init__(self, response):
        if type(response) == str:
            try:
                self.response = json.loads(response)
            except json.JSONDecodeError as decodeErr:
                print ("JSON Decode Error:",decodeErr) 
                raise Exception("no (valid) response from broker")
        else:
            raise Exception("response must be of type str")
            
    def __repr__(self):
        description = "Broker Response"
        #find number of results
        try:
            ntot = self.response["response"]["numFound"]            
        except:
            ntot = 0
            pass
        try:
            ndocs = len(self.response["response"]["docs"])            
        except:
            ndocs = 0
            pass
        description += "\n- documents: {0} from {1}".format(ndocs, ntot)
        if ndocs>0:
            try:
                start = self.response["response"]["start"]            
            except:
                start = 0
            description += ", starting at {0}".format(start)
        if "facet_counts" in self.response:    
            description += "\n- facet_counts:"
            if ("facet_queries" in self.response["facet_counts"]) & bool(self.response["facet_counts"]["facet_queries"]): 
                description += "\n  - facet_queries ({0})".format(len(self.response["facet_counts"]["facet_queries"]))
            if ("facet_fields" in self.response["facet_counts"]) & bool(self.response["facet_counts"]["facet_fields"]): 
                description += "\n  - facet_fields ({0})".format(len(self.response["facet_counts"]["facet_fields"]))
            if ("facet_ranges" in self.response["facet_counts"]) & bool(self.response["facet_counts"]["facet_ranges"]): 
                description += "\n  - facet_ranges ({0})".format(len(self.response["facet_counts"]["facet_ranges"]))
            if ("facet_intervals" in self.response["facet_counts"]) & bool(self.response["facet_counts"]["facet_intervals"]): 
                description += "\n  - facet_intervals ({0})".format(len(self.response["facet_counts"]["facet_intervals"]))
            if ("facet_heatmaps" in self.response["facet_counts"]) & bool(self.response["facet_counts"]["facet_heatmaps"]): 
                description += "\n  - facet_heatmaps ({0})".format(len(self.response["facet_counts"]["facet_heatmaps"]))
        return(description)    
    
    def raw(self): 
        return(self.response)
    
    def numFound(self): 
        try:
            return(self.response["response"]["numFound"])
        except:
            return(None)
        
    def start(self): 
        try:
            return(self.response["response"]["start"])
        except:
            return(None)    
    
    def docs(self): 
        try:
            return(pd.DataFrame(self.response["response"]["docs"]))
        except:
            return(None)
        
        
        
